package com.qipeng.checkstyle.checks;

import com.puppycrawl.tools.checkstyle.api.AbstractCheck;
import com.puppycrawl.tools.checkstyle.api.DetailAST;
import com.puppycrawl.tools.checkstyle.api.TokenTypes;

/**
 * Logger 对象定义的检查类。
 *
 * @author tieguanyin
 */
@SuppressWarnings("all")
public class LoggerNameCheck extends AbstractCheck {

  @Override
  public int[] getDefaultTokens() {
    return getAcceptableTokens();
  }

  @Override
  public int[] getAcceptableTokens() {
    return new int[]{TokenTypes.VARIABLE_DEF};
  }

  @Override
  public int[] getRequiredTokens() {
    return getAcceptableTokens();
  }

  @Override
  public void visitToken(DetailAST ast) {
    DetailAST type = ast.findFirstToken(TokenTypes.TYPE).getFirstChild();
    if ("Logger".equals(type.getText())) {
      final DetailAST modifiersAst = ast.findFirstToken(TokenTypes.MODIFIERS);
      final boolean isFinal = modifiersAst.branchContains(TokenTypes.FINAL);
      final boolean isStatic = modifiersAst.branchContains(TokenTypes.LITERAL_STATIC);

      if (!isFinal || !isStatic) {
        log(type.getLineNo(), type.getColumnNo(), "Logger 必须以 static final 修饰");
      }

      DetailAST variableName = ast.findFirstToken(TokenTypes.IDENT);
      if (!"logger".equals(variableName.getText())) {
        log(type.getLineNo(), type.getColumnNo(), "Logger 必须以 logger 命名");
      }
    }
  }

}
